/******************************************************************************
  Copyright (c) 1992 Xerox Corporation.  All rights reserved.
  Portions of this code were written by Stephen White, aka ghond.
  Use and copying of this software and preparation of derivative works based
  upon this software are permitted.  Any distribution of this software or
  derivative works must comply with all applicable United States export
  control laws.  This software is made available AS IS, and Xerox Corporation
  makes no warranty about the software, its performance or its conformity to
  any specification.  Any person obtaining a copy of this software is requested
  to send their name and post office or electronic mail address to:
    Pavel Curtis
    Xerox PARC
    3333 Coyote Hill Rd.
    Palo Alto, CA 94304
    Pavel@Xerox.Com
 *****************************************************************************/

#ifndef Tasks_H
#define Tasks_H 1

#include "config.h"
#include "execute.h"
#include "structures.h"

typedef struct {
    void       *ptr;
} task_queue;

typedef enum { TASK_INPUT, TASK_FORKED, TASK_SUSPENDED } task_kind;

extern void	set_task_kind(task_kind kind);

extern task_queue	new_task_queue(Objid player);
extern void		free_task_queue(task_queue q);

extern void		set_input_task_holding(task_queue, int);

extern void		enqueue_input_task(task_queue, const char *);
extern Var		enqueue_forked_task(Program *program, activation a,
					    Var *rt_env, unsigned f_index, 
					    unsigned after_seconds);
extern enum error	enqueue_suspended_task(vm the_vm, void *data);
				/* data == &(int after_seconds) */
extern enum error	make_reading_task(vm the_vm, void *data);
				/* data == &(Objid connection) */

extern Var		read_input_now(Objid connection);

extern int		next_task_start(void);
extern void		run_ready_tasks(void);
extern enum outcome	run_server_task(Objid player, Objid what,
					const char *verb, Var args,
					const char *argstr, Var *result);
extern enum outcome	run_server_task_setting_id(Objid player, Objid what,
						   const char *verb, Var args,
						   const char *argstr,
						   Var *result, int *task_id);

extern int		current_task_id;
extern int		last_input_task_id(Objid player);

extern void		write_task_queue(FILE *);
extern int		read_task_queue(FILE *);

#endif /* !Tasks_H */

/* $Log: tasks.h,v $
 * Revision 1.5  1992/10/23  23:03:47  pavel
 * Added copyright notice.
 *
 * Revision 1.4  1992/10/21  03:02:35  pavel
 * Converted to use new automatic configuration system.
 *
 * Revision 1.3  1992/09/08  21:56:27  pjames
 * Removed some procedures which are no longer external.
 *
 * Revision 1.2  1992/08/10  16:48:00  pjames
 * Changed Parse_Info to activation in enqueue_forked_task argument list,
 * because all important information is now stored in activation.
 *
 * Revision 1.1  1992/07/20  23:23:12  pavel
 * Initial RCS-controlled version.
 */
